@@ -6,7 +6,7 @@ use warnings FATAL => 'all';
 use parent qw(Exporter);
 use Carp qw(croak);
 
-BEGIN { our @EXPORT_OK = qw(diff dselect dsplit patch) }
+our @EXPORT_OK = qw(diff dselect dsplit patch);
 
 sub _validate_meta($) {
     my $d = shift;
@@ -24,11 +24,11 @@ Struct::Diff - Recursive diff tools for nested perl structures
 
 =head1 VERSION
 
-Version 0.55
+Version 0.58
 
 =cut
 
-our $VERSION = '0.55';
+our $VERSION = '0.58';
 
 =head1 SYNOPSIS
 
@@ -44,11 +44,11 @@ our $VERSION = '0.55';
     # @items == ({z => {A => 33}});
 
     $href = dsplit($diff);                          # divide diff
-    # $dsplit->{a} not exists                       # because unchanged omitted and all other items originated from $b
+    # $dsplit->{a} not exists                       # unchanged omitted, other items originated from $b
     # $dsplit->{b} == {x => [{y => 9}],z => 33};
 
     patch($a, $diff);
-    # $a now equal to $b
+    # $a now equal to $b by structure and data
 
 =head1 EXPORT
 
@@ -58,7 +58,7 @@ Nothing exports by default
 
 =head2 diff
 
-Returns HASH reference to diff between two passed things. Beware when
+Returns HASH reference to recursive diff between two passed things. Beware when
 changing diff: some of it's substructures are links to original structures.
 
     $diff = diff($a, $b, %opts);
@@ -72,19 +72,19 @@ Diff's keys shows status of each item in passed structures.
 
 =item A
 
-stands for 'added' (exists only in second structure), it's value - added item.
+Stands for 'added' (exists only in second structure), it's value - added item.
 
 =item D
 
-means 'different' and contains subdiff.
+Means 'different' and contains subdiff.
 
 =item I
 
-shows index for changed items (in arrays only).
+Shows index for changed item (arrays only).
 
 =item N
 
-is a new value for changed item.
+Is a new value for changed item.
 
 =item O
 
@@ -92,11 +92,11 @@ Alike 'N', 'O' is a changed item's old value.
 
 =item R
 
-similar for 'A', but for removed items.
+Similar for 'A', but for removed items.
 
 =item U
 
-represent 'unchanged' items - common for both structures.
+Represent 'unchanged' items - common for both structures.
 
 =back
 
@@ -106,11 +106,11 @@ represent 'unchanged' items - common for both structures.
 
 =item noX
 
-Where X is a status (A, N, O, R, U); such status will be omitted
+Where X is a status (A, N, O, R, U); such status will be omitted.
 
 =item trimR
 
-Drop removed item's data
+Drop removed item's data.
 
 =back
 
@@ -161,8 +161,8 @@ sub diff($$;@) {
         my $s = { map { $_, 1 } map { keys %{$_} } exists $d->{'D'} ? @{$d->{'D'}} : { 'U' => 1 } };
         delete $s->{'I'}; # ignored -- not a status
 
-        if (keys %{$s} == 1 and not $hidden) { # all have one state - drop D and return native state
-            my ($n) = (keys(%{$s}))[0];
+        if (keys %{$s} == 1 and not ($hidden or exists $s->{'D'})) { # all has same status - drop D and return as is
+            my $n = (keys(%{$s}))[0];
             map { $_ = $_->{$n} } @{$d->{'D'}};
             $d->{$n} = delete $d->{'D'};
         }
@@ -196,7 +196,7 @@ sub diff($$;@) {
             for my $s (keys %{$d}) {
                 next if ($s eq 'D');
                 map { $d->{'D'}->{$_}->{$s} = delete $d->{$s}->{$_} } keys %{$d->{$s}};
-                delete $d->{$s} unless ($s eq 'D');
+                delete $d->{$s};
             }
         }
     } else { # treat others as scalars
@@ -230,7 +230,7 @@ specified states will be returned if opt exists, but not defined or is an empty
 =item states
 
 Expects hash with desired states as keys with values in some true value. Items with all states will be returned if
-opt not defined
+opt not defined.
 
 =back
 
@@ -270,10 +270,10 @@ sub dselect(@) {
 
 =head2 dsplit
 
-Divide diff to pseudo original structures.
+Divide diff to pseudo original structures
 
     $structs = dsplit($diff);
-    # $structs->{'a'} - now contains items originated from original $a
+    # $structs->{'a'} - now contains items originated from $a
     # $structs->{'b'} - same for $b
 
 =cut
@@ -287,70 +287,26 @@ sub dsplit($) {
     if (exists $d->{'D'}) {
         if (ref $d->{'D'} eq 'ARRAY') {
             for my $di (@{$d->{'D'}}) {
-                my ($ts) = dsplit($di);
+                my $ts = dsplit($di);
                 push @{$s->{'a'}}, $ts->{'a'} if (exists $ts->{'a'});
                 push @{$s->{'b'}}, $ts->{'b'} if (exists $ts->{'b'});
             }
-        } elsif (ref $d->{'D'} eq 'HASH') {
+        } else { # HASH
             for my $key (keys %{$d->{'D'}}) {
-                my ($ts) = dsplit($d->{'D'}->{$key});
+                my $ts = dsplit($d->{'D'}->{$key});
                 $s->{'a'}->{$key} = $ts->{'a'} if (exists $ts->{'a'});
                 $s->{'b'}->{$key} = $ts->{'b'} if (exists $ts->{'b'});
             }
         }
-    }
-
-    if (exists $d->{'U'}) {
-        if (ref $d->{'U'} eq 'ARRAY') {
-            push @{$s->{'a'}}, @{$d->{'U'}};
-            push @{$s->{'b'}}, @{$d->{'U'}};
-        } elsif (ref $d->{'U'} eq 'HASH') {
-            $s->{'a'} = defined $s->{'a'} ? { %{$s->{'a'}}, %{$d->{'U'}} } : { %{$d->{'U'}} };
-            $s->{'b'} = defined $s->{'b'} ? { %{$s->{'b'}}, %{$d->{'U'}} } : { %{$d->{'U'}} };
-        } else {
-            croak "Duplicates with different status" if (defined $s->{'a'} or defined $s->{'b'});
-            $s->{'a'} = $s->{'b'} = $d->{'U'};
-        }
-    }
-
-    if (exists $d->{'A'}) {
-        if (ref $d->{'A'} eq 'ARRAY') {
-            push @{$s->{'b'}}, @{$d->{'A'}};
-        } elsif (ref $d->{'A'} eq 'HASH') {
-            $s->{'b'} = defined $s->{'b'} ? { %{$s->{'b'}}, %{$d->{'A'}} } : { %{$d->{'A'}} };
-        } else {
-            $s->{'b'} = $d->{'A'};
-        }
-    }
-
-    if (exists $d->{'N'}) {
-        if (ref $d->{'N'} eq 'ARRAY') {
-            push @{$s->{'b'}}, $d->{'N'}, splice(@{$s->{'b'}}, $d->{'I'});
-        } elsif (ref $d->{'A'} eq 'HASH') {
-            $s->{'b'} = defined $s->{'b'} ? { %{$s->{'b'}}, %{$d->{'N'}} } : { %{$d->{'N'}} };
-        } else {
-            $s->{'b'} = $d->{'N'};
-        }
-    }
-
-    if (exists $d->{'O'}) {
-        if (ref $d->{'O'} eq 'ARRAY') {
-            push @{$s->{'b'}}, $d->{'O'}, splice(@{$s->{'b'}}, $d->{'I'});
-        } elsif (ref $d->{'O'} eq 'HASH') {
-            $s->{'a'} = defined $s->{'a'} ? { %{$s->{'a'}}, %{$d->{'O'}} } : { %{$d->{'O'}} };
-        } else {
-            $s->{'a'} = $d->{'O'};
-        }
-    }
-
-    if (exists $d->{'R'}) {
-        if (ref $d->{'R'} eq 'ARRAY') {
-            push @{$s->{'a'}}, @{$d->{'R'}};
-        } elsif (ref $d->{'R'} eq 'HASH') {
-            $s->{'a'} = defined $s->{'a'} ? { %{$s->{'a'}}, %{$d->{'R'}} } : { %{$d->{'R'}} };
-        } else {
-            $s->{'a'} = $d->{'R'};
-        }
+    } elsif (exists $d->{'U'}) {
+        $s->{'a'} = $s->{'b'} = $d->{'U'};
+    } elsif (exists $d->{'A'}) {
+        $s->{'b'} = $d->{'A'};
+    } elsif (exists $d->{'R'}) {
+        $s->{'a'} = $d->{'R'};
+    } else {
+        $s->{'b'} = $d->{'N'} if (exists $d->{'N'});
+        $s->{'a'} = $d->{'O'} if (exists $d->{'O'});
     }
 
     return $s;
@@ -358,7 +314,7 @@ sub dsplit($) {
 
 =head2 patch
 
-Apply diff.
+Apply diff
 
     patch($a, $diff);
 
@@ -386,7 +342,7 @@ sub patch($$) {
                 }
                 pop @{$s} if (exists $d->{'D'}->[$i]->{'R'});
             }
-        } else {
+        } else { # HASH
             for my $k (keys %{$d->{'D'}}) {
                 next if (exists $d->{'D'}->{$k}->{'U'});
                 if (exists $d->{'D'}->{$k}->{'D'} or exists $d->{'D'}->{$k}->{'N'}) {
@@ -411,7 +367,7 @@ Struct::Diff fails on structures with loops in references. has_circular_ref from
 to detect such structures.
 
 Only scalars, refs to scalars, ref to arrays and ref to hashes correctly traversed. All other data types compared
-by their reference.
+by their references.
 
 No object oriented interface provided.
 
@@ -455,10 +411,12 @@ L<http://search.cpan.org/dist/Struct-Diff/>
 
 =head1 SEE ALSO
 
-L<Data::Diff>
+L<Data::Diff>, L<Data::Difference>
 
 L<Array::Diff>, L<Array::Compare>, L<Algorithm::Diff>, L<Data::Compare>, L<Hash::Diff>, L<Test::Struct>,
-L<Struct::Compare> L<Data::Structure::Util>
+L<Struct::Compare>
+
+L<Data::Structure::Util>, L<Struct::Path>, L<Struct::Path::PerlStyle>
 
 =head1 LICENSE AND COPYRIGHT
 
